#!/system/bin/sh

cd $(dirname $0)

for arg in "$@"; do
    case "$arg" in 
        0.3 | 0.5 | 0.7 | disable )
            settings put global downscale "$arg"
            ;;
        60 | 90 | 120 )
            settings put global fps "$arg"
            ;;
    esac
done

for arg  in "$@"; do
    case "$arg" in
          "-dbs")
            for debug in $(getprop | grep -F '[debug.' | cut -f 2 -d [ | cut -f 1 -d ]); do
                setprop "$debug" ""
            done
            ;;
          "-svk")
            if ls /system/lib/libvulkan.so > /dev/null 2>&1; then
                setprop debug.hwui.renderer skiavk
                setprop debug.renderengine.backend skiavkthreaded
                echo "Set Render with » $(getprop debug.hwui.renderer)"
                echo "Ser Render engine with » $(getprop debug.renderengine.backend)"
            else
                echo "[ Device Not Support Vulkan Render ! ]"
            fi   
            ;;
          "-vk")
            if ls /system/lib/libvulkan.so > /dev/null; then
                setprop debug.renderengine.backend vulkan
            	setprop debug.hwui.renderer vulkan                
                echo "Set Render with » $(getprop debug.hwui.renderer)"
                echo "Set Render engine with » $(getprop debug.renderengine.backend)"
           else
          	echo "[ Device Not Support Vulkan Render ! ]"    
           fi
            ;;
    esac
done

game=$(pm list packages | grep -i "mobile" | grep -i "legends" | sed  's/package://g')
id=($(cmd package dump "$game" | awk '/MAIN/{getline; print $2}'))
status=$(am get-standby-bucket "$game")
fps=$(settings get global fps)
downscale=$(settings get global downscale)
size=$(wm size)
density=$(wm density)
width=$(echo $size | cut -d 'x' -f 1)
height=$(echo $size | cut -d 'x' -f 2)
cmdgame=$(cmd -l | grep -o "game")
t=("reset-throttling" "reset-all-throttling");

cmd activity set-inactive "$game" false
cmd appops set "$game" INSTANT_APP_START_FOREGROUND allow
cmd appops set "$game" RUN_ANY_IN_BACKGROUND allow
cmd appops set "$game" RUN_IN_BACKGROUND allow
cmd appops set "$game" START_FOREGROUND allow
cmd app_hibernation set-state "$game" false
cmd dropbox remove-low-priority "$game"

set_size_and_density() {
    scale=$1
    new_width=$(awk "BEGIN {print $width * $scale}")
    new_height=$(awk "BEGIN {print $height * $scale}")
    new_size="${new_width%x}x${new_height%x}"
    new_density=$(awk "BEGIN {print $density * $scale}")
    wm size "$new_size"
    wm density "$new_density"
}

for arg in "$@"; do
    case "$arg" in
        "175")
            set_size_and_density 1.75
            ;;
        "150")
            set_size_and_density 1.50
            ;;
        "125")
            set_size_and_density 1.25
            ;;
        "100")
            wm size reset
            wm density reset
            ;;
        "75")
            set_size_and_density 0.75
            ;;
        "50")
            set_size_and_density 0.50
            ;;
        "30")
            set_size_and_density 0.30
            ;;
    esac
done

for a in $(pm list packages -U|grep -v $game|cut -f3 -d:); do
    pm trim-caches 99G "$a" &
done 

armeabi_v7a() {
    apk_path=$(pm path $game | sed 's/package://g')
    armeabi_v7a_apps=()

    if [ -n "$apk_path" ]; then
        if unzip -l "$apk_path" | grep -q "lib/armeabi-v7a/"; then
            armeabi_v7a_apps+=("$game")
        fi
    fi

    echo "${armeabi_v7a_apps[@]}"
}

compile () {
cmd package compile -r bg-dexopt -m everything -p PRIORITY_INTERACTIVE_FAST -f "$game" --primary-dex --secondary-dex --include-dependencies --full
    if [ $? -eq 0 ]; then
        echo "[ $game Compiled ! ]"
    else
        pm compile -compile -r bg-dexopt -m everything -p PRIORITY_INTERACTIVE_FAST -f "$game" --primary-dex --secondary-dex --include-dependencies --full
            if [ $? -eq 0 ]; then
                echo "[ $game Compiled ! ]"
            else
                 echo "[ Can't Compile App or packagename not found! ]"
            fi
     fi
}
cmp=false

launch_app () {
    abi=($(armeabi_v7a))

    if [ ${#abi[@]} -gt 0 ]; then
        am start -S --user 0 "${id[0]}" --es --no-window-animation --abi ARMEABI-V7A
        if [ $? -eq 0 ]; then
            cmd notification post -S bigtext -t 'ML QL · Laxeron' 'Executed' 'Starting app with Armeabi-v7a!' > /dev/null 2>&1 &
        else
            am start -S --user 0 "${id[0]}" --es --no-window-animation
                if [ $? -eq 0 ]; then
                    cmd notification post -S bigtext -t 'ML QL · Laxeron' 'Executed' 'Starting APP, Enjoy your games!' > /dev/null 2>&1 &
                fi
        fi
    else
        am start -S --user 0 "${id[0]}" --es --no-window-animation
            if [ $? -eq 0 ]; then
                cmd notification post -S bigtext -t 'ML QL · Laxeron' 'Executed' 'Starting APP, Enjoy your games!' > /dev/null 2>&1 &
            fi
    fi
}

launch=true

for arg  in "$@"; do
    case "$arg" in
           "-sk")
               launch=false
            ;;
           "-cmp")
              cmp=true
            ;;
    esac
done

echo "Game Discovered as » $game"
echo "Current Render » $(getprop debug.hwui.renderer)"
echo "Current Render engine » $(getprop debug.renderengine.backend)"
echo "Current fps » $(settings get global fps)"
echo "Current Downscale » $(settings get global downscale)"

( dumpsys deviceidle enable | dumpsys deviceidle force-idle |dumpsys deviceidle step deep ) > /dev/null

if dumpsys deviceidle | grep -q "$game"; then
    echo "[ $game already in whitelist ]"
else
    echo "[ $game is not listed ]"
    cmd deviceidle whitelist +$game > /dev/null
    echo "[ $game Added to Whitelist. ]"
fi

if [ -n "$cmdgame" ]; then
     cmd game mode performance $game
     cmd game set --mode performance --downscale "$downscale" --fps "$fps" --user 0 "$game" > /dev/null
          if [ $? -eq 0 ]; then
               echo "[ Cmd Game Applied! ]"
          else
               cmd game set --mode performance --downscale "$downscale" --fps "$fps" "$game" > /dev/null
                   if [ $? -eq 0 ]; then
                       echo "[ Cmd Game Applied ]"
                   else
                       echo "[ Can't Apply Cmd Game ]"
                   fi
          fi
fi

device_config delete game_overlay "$game" > /dev/null
  if [ $? -eq 0 ]; then  
     device_config put game_overlay "$game" mode=2,fps="$fps",downscaleFactor="$downscale"
  else
     echo "[ Can't Apply game_overlay or Error ! ]"
     echo ""
  fi

cmd package compile -m quicken -f com.android.systemui > /dev/null
    if [ $? -eq 0 ]; then
        echo "[ SystemUi is Compiled ! ]"
    else
        pm compile -m quicken -f com.android.systemui > /dev/null
        if [ $? -eq 0 ]; then
            echo "[ SystemUi is Compiled ! ]"
        else
            echo "[ Can't Compile App or packagename not found! ]"
        fi
    fi

if [ "$status" -ne 10 ]; then
    am set-standby-bucket "$game" 10
    if [ $? -eq 0 ]; then
        echo "[ $game now in High Standby Mode ]"
    else
        echo "[ Failed to set $game in Standby Mode! ]"
    fi
else
    echo "[ $game is already in Standby Mode! ]"
fi

for a in "${t[@]}"; do
    cmd shortcut "$a" "$game" > /dev/null
        if [ $? -eq 0 ]; then
            echo "[ App Throttle Reseted ! ]"
        else
            echo "[ Failed to reset App Throttle ! ]"
        fi
done

if $cmp ; then
    results+=($(compile))
fi

if $launch ; then
    results+=($(launch_app))   
fi

if [ -n "$(dumpsys activity top | grep "$game")" ] ; then
   am clear-watch-heap $game 
       if [ $? -eq 0 ]; then
           echo "[ $game heap cleared ! ]"
       else
           echo "[ Can't clear $game heap ]"
       fi
fi

am send-trim-memory --user 0 com.android.systemui RUNNING_CRITICAL

setprop debug.cpurend.vsync false
setprop debug.egl.force_msaa false
setprop debug.sf.hw 1
setprop debug.egl.hw 1
setprop debug.egl.sync 0
setprop debug.composition.type gpu

device_config put activity_manager_native_boot use_freezer true

cmd power set-adaptive-power-saver-enabled false
cmd power set-fixed-performance-mode-enabled true
cmd power set-mode 0
cmd thermalservice override-status 0
cmd looper_stats disable

settings put global cached_apps_freezer 1
settings delete system thermal_limit_refresh_rate > /dev/null
settings put system peak_refresh_rate 1
settings put system user_refresh_rate 1
settings put secure speed_mode_enable 1
settings put system speed_mode 1
settings put global window_animation_scale 0.3
settings put global transition_animation_scale 0.3
settings put global animator_duration_scale 0.3
